@@ -7,7 +7,7 @@ Tools for analyzing UEFI firmware using radare2
 import binascii
 import json
 import os
-from typing import Any, Dict, List, Optional, Set, Tuple
+from typing import Any, Dict, List, Optional, Tuple
 
 import yaml
 
@@ -478,8 +478,37 @@ class UefiScanner:
     def __init__(self, uefi_analyzer: UefiAnalyzer, uefi_rules: List[UefiRule]):
         self._uefi_analyzer: UefiAnalyzer = uefi_analyzer
         self._uefi_rules: List[UefiRule] = uefi_rules
+        self._valid_rules: bool = self._check_rules()
+        if not self._valid_rules:
+            raise UefiScannerError(
+                "Invalid rule format. Visit https://github.com/binarly-io/FwHunt to find the latest version of the rules format."
+            )
         self._results: Optional[List[UefiScannerRes]] = None
 
+    def _check_rule(self, rule: UefiRule):
+        variants: Optional[Dict[str, UefiRuleVariant]] = None
+        try:
+            variants = rule.variants
+        except KeyError:
+            return False
+
+        if not isinstance(variants, dict):
+            return False
+
+        for label in variants:
+            if not isinstance(label, str):
+                return False
+            if not isinstance(variants[label], UefiRuleVariant):
+                return False
+
+        return True
+
+    def _check_rules(self):
+        for rule in self._uefi_rules:
+            if not self._check_rule(rule):
+                return False
+        return True
+
     def _and_strings(self, strings: List[str]) -> bool:
         res = True
         for string in strings:
@@ -829,7 +858,12 @@ class UefiScanner:
     def _get_bounds(self, insns: List[Dict[str, Any]]) -> Tuple:
         """Get function end address"""
 
-        funcs = list(self._uefi_analyzer._rz.cmdj("aflqj"))
+        funcs = list(
+            filter(
+                lambda addr: addr,
+                [addr.get("offset", None) for addr in self._uefi_analyzer.functions],
+            )
+        )
         funcs.sort()
 
         start = insns[0].get("offset", None)
@@ -866,14 +900,14 @@ class UefiScanner:
         """Recursively traverse the function and find the boundaries of all child functions"""
 
         self._uefi_analyzer._rz.cmd(f"s {start_addr:#x}")
-        self._uefi_analyzer._rz.cmd(f"af")
+        self._uefi_analyzer._rz.cmd("af")
 
         insns = self._uefi_analyzer._rz.cmd("pdrj")
         # prevent error messages to sys.stderr from rizin:
         # https://github.com/rizinorg/rz-pipe/blob/0f7ac66e6d679ebb03be26bf61a33f9ccf199f27/python/rzpipe/open_base.py#L261
         try:
             insns = json.loads(insns)
-        except (ValueError, KeyError, TypeError) as _:
+        except (ValueError, KeyError, TypeError):
             return False
 
         # append function bounds
@@ -900,7 +934,7 @@ class UefiScanner:
                 if address not in self._rec_addrs:
                     self._rec_addrs.append(address)
                     self._get_bounds_rec(address, depth, debug)
-            except ValueError as _:
+            except ValueError:
                 continue
 
         return True
@@ -951,7 +985,7 @@ class UefiScanner:
             elif c.child_sw_smi_handlers:
                 handlers = self._uefi_analyzer.child_swsmi_handlers
             else:
-                raise UefiScannerError(f"The search place is incorrect")
+                raise UefiScannerError("The search place is incorrect")
             search_res = False
             for handler in handlers:
                 search_res = self._code_scan_rec(handler.address, c.pattern)
@@ -973,7 +1007,7 @@ class UefiScanner:
             elif c.child_sw_smi_handlers:
                 handlers = self._uefi_analyzer.child_swsmi_handlers
             else:
-                raise UefiScannerError(f"The search place is incorrect")
+                raise UefiScannerError("The search place is incorrect")
             search_res = False
             for handler in handlers:
                 search_res = self._code_scan_rec(handler.address, c.pattern)
